@@ -78,7 +78,8 @@ class XRD():
         3x3 representation -> 1x6 (a, b, c, alpha, beta, gamma)
         """
 
-        rec_matrix = crystal.get_reciprocal_cell()
+        #rec_matrix = crystal.get_reciprocal_cell()
+        rec_matrix = crystal.cell.reciprocal()
         d_min = self.wavelength/np.sin(self.max2theta/2)/2
 
         # This block is to find the shortest d_hkl
@@ -329,7 +330,9 @@ class XRD():
             plt.close()
 
 
-    def plotly_pxrd(self, profile='gaussian', minimum_I = 0.01, height=450, html=None):
+    def plotly_pxrd(self, profile='gaussian', minimum_I=0.01, res=0.02, \
+            FWHM=0.1, height=450, html=None):
+
         import plotly.graph_objects as go
 
         """
@@ -356,7 +359,7 @@ class XRD():
         if profile is None:
             fig = go.Figure(data=[trace1])
         else:
-            spectra = self.get_profile(method=profile)
+            spectra = self.get_profile(method=profile, res=res, user_kwargs={"FWHM": FWHM})
             trace2 = go.Scatter(x=spectra[0], y=spectra[1], name='Profile: ' + profile)
             fig = go.Figure(data=[trace2, trace1])
 
@@ -398,7 +401,7 @@ class Profile:
         user_kwargs (dict): The parameters for the profiling method.
     """
 
-    def __init__(self, method='mod_pseudo-voigt', res = 0.02, user_kwargs=None):
+    def __init__(self, method='mod_pseudo-voigt', res=0.02, user_kwargs=None):
 
         self.method = method
         self.user_kwargs = user_kwargs
@@ -415,7 +418,7 @@ class Profile:
                       'eta_l': 0.611844,
                      }
         elif method in ['gaussian', 'lorentzian', 'pseudo-voigt']:
-            _kwargs = {'FWHM': 0.05}
+            _kwargs = {'FWHM': 0.1}
 
         else:
             msg = method + " isn't supported."
@@ -542,10 +545,10 @@ class Similarity():
         Npts = len(self.fx)
         d = self.fx[1] - self.fx[0]
 
-        self.S = similarity_calculate(self.r, w, d, Npts, self.fy, self.gy)
+        self.value = similarity_calculate(self.r, w, d, Npts, self.fy, self.gy)
 
     def __str__(self):
-        s = "The similarity between two PXRDs is {:.4f}".format(self.S)
+        s = "The similarity between two PXRDs is {:.4f}".format(self.value)
         return s
 
     def __repr__(self):
@@ -599,7 +602,7 @@ class Similarity():
         residuals = self.gy - self.fy
         fig1.add_axes((.1,.1,.8,.2))
         plt.plot(self.gx, residuals, '.r', markersize = 0.5)
-        plt.title("{:6f}".format(self.S))
+        plt.title("{:6f}".format(self.value))
 
         if filename is None:
             plt.show()
