@@ -14,12 +14,23 @@ pyxtal_verbosity = 1  # constant for printx function
 # Max number of atoms per molecule before using fast distance check
 max_fast_mol_size = 30
 letters = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
+ltype_keywords = [
+                  "triclinic", "Triclinic",
+                  "monoclinic", "Monoclinic",
+                  "orthorhombic", "Orthorhombic",
+                  "tetragonal", "Tetragonal",
+                  "trigonal", "Trigonal",
+                  "hexagonal", "Hexagonal",
+                  "cubic", "Cubic",
+                  "spherical", "Spherical",
+                  "ellipsoidal", "Ellipsoidal",
+                 ]
 
 logo = """#############################################################
-#             ______       _    _          _   	            #
+#             ______       _    _          _                #
 #            (_____ \     \ \  / /        | |               #
-#             _____) )   _ \ \/ / |_  ____| |  	            #
-#            |  ____/ | | | )  (|  _)/ _  | | 	            #
+#             _____) )   _ \ \/ / |_  ____| |               #
+#            |  ____/ | | | )  (|  _)/ _  | |               #
 #            | |    | |_| |/ /\ \ |_( (_| | |___            #
 #            |_|     \__  /_/  \_\___)__|_|_____)           #
 #                   (____/                                  #
