@@ -17,45 +17,55 @@ from monty.serialization import loadfn
 
 bonds = loadfn(resource_filename("pyxtal", "database/bonds.json"))
 
+def in_merged_coords(wp, pt, pts, cell):
+    """
+    Whether or not the pt in within the pts
+    """
+    (c, s) = pt
+    for pt0 in pts:
+        (c0, s0) = pt0
+        if s == s0 and wp.are_equivalent_pts(c, c0, cell):
+            #print(c, c0, 'equivalent')
+            return True
+    return False
+
+
 def write_cif(struc, filename=None, header="", permission='w', sym_num=None, style='mp'):
     """
     Export the structure in cif format
-    The default setting for _atom_site follows the materials project cif 
+    The default setting for _atom_site follows the materials project cif
 
     Args:
         struc: pyxtal structure object
-        filename: path of the structure file 
+        filename: path of the structure file
         header: additional information
         permission: write(`w`) or append(`a+`) to the given file
         sym_num: the number of symmetry operations, None means writing all symops
         style: `icsd` or `mp` (used in pymatgen)
 
     """
-    if sym_num is None:
-        l_type = struc.group.lattice_type
-        symbol = struc.group.symbol
-        number = struc.group.number
-        G1 = struc.group.Wyckoff_positions[0]
-    else: #P1 symmetry
-        l_type = 'triclinic'
-        symbol = 'P1'
-        number = 1
-        G1 = Group(1).Wyckoff_positions[0]
-
-    if hasattr(struc, 'mol_sites'):
+    if struc.molecular:
         sites = struc.mol_sites
         molecule = True
     else:
         sites = struc.atom_sites
         molecule = False
 
-    change_set = False
-    if number in [7, 14, 15]:
-        if hasattr(struc, 'diag') and struc.diag:
-            symbol = struc.group.alias 
-            G1.diagonalize_symops()
-            change_set = True
-    
+    if sym_num is None:
+        l_type = struc.group.lattice_type
+        number = struc.group.number
+        G1 = struc.group[0]
+        if G1.is_standard_setting():
+            symbol = struc.group.symbol
+        else:
+            symbol = sites[0].wp.get_hm_symbol()
+
+    else: #P1 symmetry
+        l_type = 'triclinic'
+        symbol = 'P1'
+        number = 1
+        G1 = Group(1).Wyckoff_positions[0]
+
     lines = logo
     lines += 'data_' + header + '\n'
     if hasattr(struc, "energy"):
@@ -86,12 +96,7 @@ def write_cif(struc, filename=None, header="", permission='w', sym_num=None, sty
     lines += ' _symmetry_equiv_pos_site_id\n'
     lines += ' _symmetry_equiv_pos_as_xyz\n'
 
-    if not change_set:
-    #if change_set:
-        wps = G1
-    else:
-        wps = sites[0].wp.ops
-    for i, op in enumerate(wps):
+    for i, op in enumerate(G1):
         lines += "{:d} '{:s}'\n".format(i+1, op.as_xyz_string())
 
     lines += '\nloop_\n'
@@ -110,7 +115,34 @@ def write_cif(struc, filename=None, header="", permission='w', sym_num=None, sty
         letter = site.wp.letter
         if molecule:
             if sym_num is None:
-                coords, species = site._get_coords_and_species(first=True)
+                coord0s, specie0s = site._get_coords_and_species(first=True)
+                if struc.has_special_site():
+                    #print("#Check if the mul is consistent!")
+                    muls = []
+                    coords = []
+                    species = []
+                    merges = []
+
+                    for coord, specie in zip(coord0s, specie0s):
+                        _, wp, _ = G1.merge(coord, struc.lattice.matrix, 0.05)
+                        #print(coord, wp)
+                        if len(wp) > mul:
+                            if not in_merged_coords(G1,
+                                                    [coord, specie],
+                                                    merges,
+                                                    struc.lattice.matrix):
+                                coords.append(coord)
+                                species.append(specie)
+                                muls.append(len(wp))
+                                merges.append((coord, specie))
+                        else:
+                            coords.append(coord)
+                            species.append(specie)
+                            muls.append(mul)
+
+                else:
+                    coords, species = coord0s, specie0s
+                    muls = [mul] * len(coords)
             else:
                 coords = None
                 species = []
@@ -122,10 +154,12 @@ def write_cif(struc, filename=None, header="", permission='w', sym_num=None, sty
                     else:
                         coords = np.append(coords, tmp, axis=0)
                     species.extend([s.value for s in mol.species])
+                muls = [mul] * len(coords)
                 #coords, species = site._get_coords_and_species(ids=sym_num)
         else:
-            coords, species = [site.position], [site.specie]
-        for specie, coord in zip(species, coords):
+            coords, species, muls = [site.position], [site.specie], [mul]
+
+        for specie, coord, mul in zip(species, coords, muls):
             lines += '{:6s} {:6s} {:3d} '.format(specie, specie, mul)
             if style != 'mp':
                 lines += '{:s} '.format(letter)
@@ -145,7 +179,7 @@ def read_cif(filename):
     Be cautious in using it to read other cif files
 
     Args:
-        filename: path of the structure file 
+        filename: path of the structure file
 
     Return:
         pyxtal structure
@@ -198,18 +232,18 @@ def read_cif(filename):
 
 
 class structure_from_ext():
-    
-    def __init__(self, struc, ref_mols, tol=0.2, ignore_HH=False, add_H=False):
+
+    def __init__(self, struc, ref_mols, tol=0.2, ignore_HH=False, add_H=False, hn=None):
 
         """
-        extract the mol_site information from the give cif file 
+        extract the mol_site information from the give cif file
         and reference molecule
-    
-        Args: 
+
+        Args:
             struc: cif/poscar file or a Pymatgen Structure object
             ref_mols: a list of reference molecule (xyz file or Pyxtal molecule)
             tol: scale factor for covalent bond distance
-            ignore_HH: whether or not ignore the short H-H distance when checking molecule
+            ignore_HH: whether or not ignore short H-H in checking molecule
             add_H: whether or not add the H atoms
         """
 
@@ -221,7 +255,7 @@ class structure_from_ext():
             else:
                 print(type(ref_mol))
                 raise NameError("reference molecule cannot be defined")
-    
+
         if isinstance(struc, str):
             pmg_struc = Structure.from_file(struc)
         elif isinstance(struc, Structure):
@@ -235,20 +269,28 @@ class structure_from_ext():
 
         self.ref_mols = ref_mols
         self.tol = tol
-        self.diag = False
         self.add_H = add_H
 
-        sym_struc, number = get_symmetrized_pmg(pmg_struc)
-        group = Group(number)
+        sym_struc, number = get_symmetrized_pmg(pmg_struc, hn=hn)
+        if hn is None:
+            group = Group(number)
+        else:
+            group = Group(hn, use_hall=True)
+
         self.group = group
         self.wyc = group[0]
-        self.perm = [0,1,2]
 
-        molecules = search_molecules_in_crystal(sym_struc, self.tol, ignore_HH=ignore_HH)
+        molecules = search_molecules_in_crystal(sym_struc,
+                                                self.tol,
+                                                ignore_HH=ignore_HH)
 
         self.pmg_struc = sym_struc
-        self.lattice = Lattice.from_matrix(sym_struc.lattice.matrix, ltype=group.lattice_type)
+        matrix = sym_struc.lattice.matrix
+        ltype = group.lattice_type
+        self.lattice = Lattice.from_matrix(matrix, ltype=ltype)
         self.resort(molecules)
+        if len(self.ids) == 0:
+            raise RuntimeError('Cannot extract molecules')
 
     def resort(self, molecules):
         from pyxtal.operations import apply_ops, find_ids
@@ -259,78 +301,84 @@ class structure_from_ext():
         new_lat = self.lattice.matrix
         positions = np.zeros([len(molecules),3])
         for i in range(len(molecules)):
-            positions[i, :] = np.dot(molecules[i].cart_coords.mean(axis=0), inv_lat) 
+            positions[i] = np.dot(molecules[i].cart_coords.mean(axis=0), inv_lat)
+
+        self.wps = []
 
         ids = []  #id for the generator
-        mults = [] #the corresponding multiplicities
         visited_ids = []
         for id, pos in enumerate(positions):
             if id not in visited_ids:
-                ids.append(id)
-                #print("pos", pos); print(self.wyc)
                 centers = apply_ops(pos, self.wyc)
                 tmp_ids = find_ids(centers, positions)
                 visited_ids.extend(tmp_ids)
-                mults.append(len(tmp_ids))
-                #print("check", id, tmp_ids, mults)
+                if len(tmp_ids) == len(self.wyc):
+                    self.wps.append(self.wyc)
+                    ids.append(id)
+                else: #special sites
+                    for id0 in tmp_ids:
+                        p0 = positions[id0]
+                        p1, wp, _ = self.wyc.merge(p0, new_lat, 0.1)
+                        diff = p1 - p0
+                        diff -= np.round(diff)
+                        if np.abs(diff).sum() < 0.01: #sort position by mapping
+                            self.wps.append(wp)
+                            ids.append(id0) #find the right ids
+                            break
 
         # add position and molecule, print("ids", ids, mults)
         N_sites = len(ids)
         self.numMols = [0] * len(self.ref_mols)
         self.positions = []
         self.p_mols = []
-        self.wps = []
+        self.ids = []
         ids_done = []
+
+        #search for the matched molecules
         for j, mol2_ref in enumerate(self.ref_mols):
             mol2 = mol2_ref.copy()
             if self.add_H: mol2.mol.remove_species("H")
 
             for i, id in enumerate(ids):
                 mol1 = molecules[id]
+                #print(id, ids_done, len(mol2.mol), len(mol1))
+                #print(mol2.mol.to('xyz'))
                 if id not in ids_done and len(mol2.mol) == len(mol1):
                     p_mol = mol2_ref.copy() # create p_mol
                     match, mapping = compare_mol_connectivity(mol2.mol, mol1)
                     if match:
-                        self.numMols[j] += mults[i] 
                         if len(mol1) > 1:
                             # rearrange the order
                             order = [mapping[at] for at in range(len(mol1))]
-                            xyz = mol1.cart_coords[order] 
+                            xyz = mol1.cart_coords[order]
                             # add hydrogen positions here
-                            if self.add_H: xyz = self.add_Hydrogens(mol2.smile, xyz)
+                            if self.add_H:
+                                xyz = self.add_Hydrogens(mol2.smile, xyz)
                             #print(xyz)
-                            frac = np.dot(xyz, inv_lat) 
-                            xyz = np.dot(frac, new_lat) 
-                            center = p_mol.get_center(xyz) 
+                            frac = np.dot(xyz, inv_lat)
+                            xyz = np.dot(frac, new_lat)
+                            center = p_mol.get_center(xyz)
                             p_mol.reset_positions(xyz-center)
                             position = np.dot(center, np.linalg.inv(new_lat))
                         else:
                             position = np.dot(mol1.cart_coords[0], np.linalg.inv(new_lat))
                         position -= np.floor(position)
-                        #print(position); print(lat)
-                        #print(p_mol.mol.cart_coords[:10] + np.dot(position, new_lat))
-
-                        # check if molecule is on the special wyckoff position
-                        if mults[i] < len(self.wyc):
-                            #Transform it to the conventional representation
-                            if self.diag: position = np.dot(self.perm, position).T
-                            #print("molecule is on the special wyckoff position")
-                            position, wp, _ = self.wyc.merge(position, new_lat, 0.1)
-                            self.wps.append(wp)
-                            self.numMols[j] = len(wp)
-                            #print("After Merge:---"); print(position); print(wp)
-                        else:
-                            self.wps.append(self.wyc)
 
                         self.positions.append(position)
                         self.p_mols.append(p_mol)
+                        self.ids.append(j)
                         ids_done.append(id)
+                        self.numMols[j] += len(self.wps[i])
 
+        # check if some molecules cannot be matched
         if len(ids_done) < len(ids):
             for id in ids:
                 if id not in ids_done:
-                    msg = "This molecule cannot be matched to the reference molecule\n"
-                    msg += molecules[id].to('xyz')
+                    msg = "This molecule cannot be matched to the reference\n"
+                    msg += 'Molecules extracted from the structure\n'
+                    msg += molecules[id].to('xyz') + '\n'
+                    msg += "Reference molecule from smiles or xyz\n"
+                    msg += mol2.mol.to('xyz')
                     raise ReadSeedError(msg)
 
     def add_Hydrogens(self, smile, xyz):
@@ -365,7 +413,7 @@ class structure_from_ext():
         AllChem.UFFOptimizeMolecule(m1)
         m3 = AllChem.ConstrainedEmbed(m1,m2)
         conf = m3.GetConformer(0) #; print(conf.GetPositions())
-        
+
         return conf.GetPositions()
 
     def make_mol_sites(self):
@@ -374,8 +422,9 @@ class structure_from_ext():
         """
         ori = Orientation(np.eye(3))
         sites = []
-        for mol, pos, wp in zip(self.p_mols, self.positions, self.wps):
-            site = mol_site(mol, pos, ori, wp, self.lattice, self.diag)
+        for id, mol, pos, wp in zip(self.ids, self.p_mols, self.positions, self.wps):
+            site = mol_site(mol, pos, ori, wp, self.lattice)
+            site.type = id
             #print(pos)
             #print(self.lattice.matrix)
             #print([a.value for a in site.molecule.mol.species])
@@ -426,7 +475,7 @@ def search_molecules_in_crystal(struc, tol=0.2, once=False, ignore_HH=True):
         struc: Pymatgen Structure
         tol: tolerance value to check the connectivity
         once: search only one molecule or all molecules
-        ignore_HH: whether or not ignore the short H-H distance when checking molecule
+        ignore_HH: whether or not ignore the short H-H in checking molecule
 
     Returns:
         molecules: list of pymatgen molecules
@@ -438,7 +487,7 @@ def search_molecules_in_crystal(struc, tol=0.2, once=False, ignore_HH=True):
             sites_add, visited = check_one_site(struc, site0, visited)
             new_members.extend(sites_add)
         return new_members, visited
-    
+
     def check_one_site(struc, site0, visited, rmax=2.8):
         neigh_sites = struc.get_neighbors(site0, rmax)
         ids = [m.index for m in visited]
@@ -454,10 +503,11 @@ def search_molecules_in_crystal(struc, tol=0.2, once=False, ignore_HH=True):
                             (d, image) = site0.distance_and_image(site1)
                         else:
                             d = site0.distance(site1)
-                        key = "{:s}-{:s}".format(site1.specie.value, site0.specie.value)
+                        val1, val2 = site1.specie.value, site0.specie.value
+                        key = "{:s}-{:s}".format(val1, val2)
 
                         #sometime the H-H short distance is not avoidable
-                        if key == 'H-H': 
+                        if key == 'H-H':
                             if not ignore_HH:
                                 if pbc: site1.frac_coords += image
                                 sites_add.append(site1)
@@ -474,12 +524,13 @@ def search_molecules_in_crystal(struc, tol=0.2, once=False, ignore_HH=True):
                     else:
                         d = site0.distance(site1)
 
-                    key = "{:s}-{:s}".format(site1.specie.value, site0.specie.value)
+                    val1, val2 = site1.specie.value, site0.specie.value
+                    key = "{:s}-{:s}".format(val1, val2)
                     if d < bonds[key]:
                         if pbc: site1.frac_coords += image
                         sites_add.append(site1)
                         ids_add.append(site1.index)
-                    
+
         if len(sites_add) > 0:
             visited.extend(sites_add)
 
@@ -491,7 +542,7 @@ def search_molecules_in_crystal(struc, tol=0.2, once=False, ignore_HH=True):
     for id, site in enumerate(struc.sites):
         if id not in visited_ids:
             first_site = site
-            visited = [first_site] 
+            visited = [first_site]
             first_site.index = id
             n_iter, max_iter = 0, len(struc)-len(visited_ids)
             while n_iter < max_iter:
@@ -502,7 +553,7 @@ def search_molecules_in_crystal(struc, tol=0.2, once=False, ignore_HH=True):
                 n_iter += 1
                 if len(new_sites)==0:
                     break
-            
+
             coords = [s.coords for s in visited]
             coords = np.array(coords)
             numbers = [s.specie.number for s in visited]
