@@ -1,5 +1,5 @@
 """
-Module for handling molecules. 
+Module for handling molecules.
 """
 # Imports
 import os
@@ -28,23 +28,35 @@ from pyxtal.msg import ConformerError
 bonds = loadfn(resource_filename("pyxtal", "database/bonds.json"))
 molecule_collection = Collection("molecules")
 
-def find_id_from_smile(smile):
+def find_rotor_from_smile(smile):
     """
     Find the positions of rotatable bonds in the molecule.
     """
-    def cleaner(list_to_clean):
+    def cleaner(list_to_clean, neighbors):
         """
         Remove duplicate torsion from a list of atom index tuples.
         """
+        
         for_remove = []
         for x in reversed(range(len(list_to_clean))):
-            for y in reversed(range(x)):
-                ix1, ix2 = itemgetter(1)(list_to_clean[x]), itemgetter(2)(list_to_clean[x])
-                iy1, iy2 = itemgetter(1)(list_to_clean[y]), itemgetter(2)(list_to_clean[y])
-                if (ix1 == iy1 and ix2 == iy2) or (ix1 == iy2 and ix2 == iy1):
-                    for_remove.append(y)
-        clean_list = [v for i, v in enumerate(list_to_clean)
-                      if i not in set(for_remove)]
+            ix0 = itemgetter(0)(list_to_clean[x])
+            ix3 = itemgetter(3)(list_to_clean[x])
+            # for i-j-k-l, we don't want i, l are the ending members
+            # here C-C-S=O is not a good choice since O is only 1-coordinated
+            if neighbors[ix0] > 1 and neighbors[ix3] > 1:
+                for y in reversed(range(x)):
+                    ix1 = itemgetter(1)(list_to_clean[x])
+                    ix2 = itemgetter(2)(list_to_clean[x])
+                    iy1 = itemgetter(1)(list_to_clean[y])
+                    iy2 = itemgetter(2)(list_to_clean[y])
+                    if [ix1, ix2] == [iy1, iy2] or [ix1, ix2] == [iy2, iy1]:
+                        for_remove.append(y)
+            else:
+                for_remove.append(x)
+        clean_list = []
+        for i, v in enumerate(list_to_clean):
+            if i not in set(for_remove):
+                clean_list.append(v)
         return clean_list
 
     if smile in ["Cl-", "F-", "Br-", "I-", "Li+", "Na+"]:
@@ -53,51 +65,61 @@ def find_id_from_smile(smile):
         from rdkit import Chem
         smarts_torsion1="[*]~[!$(*#*)&!D1]-&!@[!$(*#*)&!D1]~[*]"
         smarts_torsion2="[*]~[^2]=[^2]~[*]" # C=C bonds
+        #smarts_torsion2="[*]~[^1]#[^1]~[*]" # C-C triples bonds, to be fixed
 
         mol = Chem.MolFromSmiles(smile)
-        pattern_tor1 = Chem.MolFromSmarts(smarts_torsion1)
-        torsion1 = cleaner(list(mol.GetSubstructMatches(pattern_tor1)))
-        pattern_tor2 = Chem.MolFromSmarts(smarts_torsion2)
-        torsion2 = cleaner(list(mol.GetSubstructMatches(pattern_tor2)))
-        tmp = cleaner(torsion1+torsion2)
+        N_atom = mol.GetNumAtoms()
+        neighbors = [len(a.GetNeighbors()) for a in mol.GetAtoms()]
+        #make sure that the ending members will be counted
+        neighbors[0] += 1
+        neighbors[-1] += 1
+        patn_tor1 = Chem.MolFromSmarts(smarts_torsion1)
+        torsion1 = cleaner(list(mol.GetSubstructMatches(patn_tor1)), neighbors)
+        patn_tor2 = Chem.MolFromSmarts(smarts_torsion2)
+        torsion2 = cleaner(list(mol.GetSubstructMatches(patn_tor2)), neighbors)
+        tmp = cleaner(torsion1+torsion2, neighbors)
         torsions = []
         for t in tmp:
             (i, j, k, l) = t
             b = mol.GetBondBetweenAtoms(j,k)
             if not b.IsInRing():
                 torsions.append(t)
+        #if len(torsions) > 6: torsions[1] = (4, 7, 10, 15)
         return torsions
 
 def generate_molecules(smile, wps=None, N_iter=4, N_conf=10, tol=0.5):
     """
     generate pyxtal_molecules from smiles codes.
-    
+
+    Args:
+        smile: smiles code
+        wps: list of wps
+        N_iter: rdkit parameter
+        N_conf: number of conformers
+        tol: rdkit parameter
+
+    Returns:
+        a list of pyxtal molecules
     """
+
     from rdkit import Chem
     from rdkit.Chem import AllChem
 
-    torsionlist = find_id_from_smile(smile)
+    torsionlist = find_rotor_from_smile(smile)
 
     def get_conformers(smile, seed):
         mol = Chem.MolFromSmiles(smile)
         mol = Chem.AddHs(mol)
         ps = AllChem.ETKDGv3()
-        ps.randomSeed=seed
-        ps.runeRmsThresh=tol
+        ps.randomSeed = seed
+        ps.runeRmsThresh = tol
         AllChem.EmbedMultipleConfs(mol, max([4, 4*len(torsionlist)]), ps)
-        #AllChem.EmbedMultipleConfs(mol, 1, ps)
         return mol
-        #engs = [c[1] for c in res]
-        #N_confs = mol.GetNumConformers()
-        #xyzs = []
-        #for conf_id in range(N_confs):
-        #    conf = mol.GetConformer(conf_id)
-        #    xyzs.append(conf.GetPositions())
-        #return xyzs
 
     m0 = pyxtal_molecule(smile+'.smi', fix=True)
     _, valid = m0.get_orientations_in_wps(wps)
     if valid:
+        #print('torsion', m0.get_torsion_angles())
         mols = [m0]
     else:
         mols = []
@@ -109,7 +131,7 @@ def generate_molecules(smile, wps=None, N_iter=4, N_conf=10, tol=0.5):
             m = m0.copy()
             xyz = m.align(conf)
             m.reset_positions(xyz)
-            m.get_symmetry()
+            m.get_symmetry(symmetrize=True)
             m.energy = res[id][1]
             _, valid = m.get_orientations_in_wps(wps)
             if valid:
@@ -119,14 +141,16 @@ def generate_molecules(smile, wps=None, N_iter=4, N_conf=10, tol=0.5):
             if add:
                 match = False
                 for mol in mols:
-                    rms, _, _ = mol.get_rmsd(xyz)
+                    rms, _ = mol.get_rmsd2(xyz, mol.mol.cart_coords)
+                    #print("rms", mol.get_torsion_angles(), rms)
                     if rms < tol:
                         match = True
                         break
                 if not match:
+                    #print(len(mols)+1, m.get_torsion_angles(xyz))
                     mols.append(m)
-            if len(mols) == N_conf:
-                break
+                    if len(mols) == N_conf:
+                        return mols
     #for m in mols:
     #    print(m.energy, m.pga.sch_symbol, len(torsionlist))
     return mols
@@ -137,9 +161,9 @@ class pyxtal_molecule:
     The added features include:
     0, parse the input
     1, estimate volume/tolerance/radii
-    2, find and store symmetry 
-    3, get the principle axis 
-    4, re-align the molecule 
+    2, find and store symmetry
+    3, get the principle axis
+    4, re-align the molecule
 
     The molecule is always centered at (0, 0, 0).
 
@@ -153,13 +177,19 @@ class pyxtal_molecule:
         tm: tolerance matrix
     """
 
-    def __init__(self, 
-                 mol=None, 
-                 symmetrize=True, 
-                 fix=False, 
-                 torsions=None,
+    def list_molecules():
+        """
+        list the internally supported molecules
+        """
+        molecule_collection.show_names()
+
+    def __init__(self,
+                 mol = None,
+                 symmetrize = True,
+                 fix = False,
+                 torsions = None,
                  seed = None,
-                 tm=Tol_matrix(prototype="molecular")):
+                 tm = Tol_matrix(prototype="molecular")):
 
         mo = None
         self.smile = None
@@ -212,25 +242,26 @@ class pyxtal_molecule:
         self.volume = self.box.volume
         self.get_symbols()
         self.get_radius()
-        self.get_tols_matrix()
+        self.tols_matrix = self.get_tols_matrix()
         xyz = self.mol.cart_coords
         self.reset_positions(xyz-self.get_center(xyz))
 
     def __str__(self):
         return '[' + self.name + ']'
 
-    def save_dict(self):
+    def save_str(self):
         """
         save the object as a dictionary
         """
-        return self.mol.as_dict()
+        d = self.mol.to(fmt='xyz')
+        return d
 
     @classmethod
-    def load_dict(cls, dicts):
+    def load_str(cls, string):
         """
         load the molecule from a dictionary
         """
-        mol = Molecule.from_dict(dicts)
+        mol = Molecule.from_str(string, fmt='xyz')
         return cls(mol)
 
     def copy(self):
@@ -248,15 +279,6 @@ class pyxtal_molecule:
 
         return pyxtal_molecule(mo, self.tm)
 
-    #def add_site_props(self, mo):
-    #    """
-    #    add site properties
-    #    """
-    #    if len(self.props) > 0:
-    #        for key in self.props.keys():
-    #            mo.add_site_property(key, self.props[key])
-    #    return mo
-
     def get_box(self, padding=None):
         """
         Given a molecule, find a minimum orthorhombic box containing it.
@@ -280,41 +302,79 @@ class pyxtal_molecule:
                 dims[i] = max([dims[i], 2.0]) #for planar molecules
             else:
                 ids = np.argsort(xyz[:, i])
-                r = Element(mol[ids[0]].species_string).vdw_radius 
-                r += Element(mol[ids[-1]].species_string).vdw_radius 
+                r = Element(mol[ids[0]].species_string).vdw_radius
+                r += Element(mol[ids[-1]].species_string).vdw_radius
                 dims[i] = max([dims[i]+r, 3.4]) #for planar molecules
         return Box(dims)
 
-    def get_box_coordinates(self, xyz, padding=0):
+    def get_box_coordinates(self, xyz, padding=0, resolution=1.0):
         """
         create the points cloud to describe the molecular box
 
         Args:
             center: molecular center position
             orientation: orientation matrix
+            padding: padding of the box
+            resolution: float in angstrom
 
         Return:
             cell: box axis
-            vertices: [8, 3] np.array, Cartesian coordinates to describe the box.
+            vertices: [N, 3] array, Cartesian coordinates to describe the box.
             center: box center
         """
         cell = self.get_principle_axes(xyz).T
         center = self.get_center(xyz) #, geometry=True)
         box = self.get_box(padding)
-        w, h, l = box.width, box.height, box.length      
+        #print(box)
+        w, h, l = box.width, box.height, box.length
         cell[0,:] *= l
         cell[1,:] *= w
         cell[2,:] *= h
-        vertices = np.array([[-1/2, -1/2, -1/2], 
-                             [1/2, -1/2, -1/2], 
-                             [1/2, 1/2, -1/2], 
-                             [-1/2, 1/2, -1/2],
-                             [-1/2, 1/2, 1/2], 
-                             [-1/2, -1/2, 1/2], 
-                             [1/2, -1/2, 1/2], 
-                             [1/2, 1/2, 1/2]])
+        x_ = np.linspace(-1/2, 1/2, int(l/resolution)+1)
+        y_ = np.linspace(-1/2, 1/2, int(w/resolution)+1)
+        z_ = np.linspace(-1/2, 1/2, int(h/resolution)+1)
+
+        #XY
+        #print(len(x_), len(y_), len(z_))
+        x, y = np.meshgrid(x_, y_, indexing='ij')
+        size = len(x.flatten())
+        xy = np.zeros([size*2, 3])
+        xy[:size,0] = x.flatten()
+        xy[size:,0] = x.flatten()
+        xy[:size,1] = y.flatten()
+        xy[size:,1] = y.flatten()
+        xy[:size,2] = -0.5
+        xy[size:,2] = 0.5
+        #print(xy.shape)
+        #print(xy)
+
+        y, z = np.meshgrid(y_, z_, indexing='ij')
+        size = len(y.flatten())
+        yz = np.zeros([size*2, 3])
+        yz[:size,1] = y.flatten()
+        yz[size:,1] = y.flatten()
+        yz[:size,2] = z.flatten()
+        yz[size:,2] = z.flatten()
+        yz[:size,0] = -0.5
+        yz[size:,0] = 0.5
+
+        x, z = np.meshgrid(x_, z_, indexing='ij')
+        size = len(z.flatten())
+        xz = np.zeros([size*2, 3])
+        xz[:size,0] = x.flatten()
+        xz[size:,0] = x.flatten()
+        xz[:size,2] = z.flatten()
+        xz[size:,2] = z.flatten()
+        xz[:size,1] = -0.5
+        xz[size:,1] = 0.5
+
+        vertices = np.zeros([len(xy)+len(yz)+len(xz), 3])
+        vertices[:len(xy),:] = xy
+        vertices[len(xy):len(xy)+len(yz),:] = yz
+        vertices[len(xy)+len(yz):,:] = xz
         vertices = vertices.dot(cell)
         vertices += center
+
         return cell, vertices, center
 
     def get_radius(self):
@@ -323,9 +383,7 @@ class pyxtal_molecule:
         """
         r_max = 0
         for coord, number in zip(self.mol.cart_coords, self.mol.atomic_numbers):
-            radius = (
-                np.sqrt(np.sum(coord * coord)) + self.tm.get_tol(number, number) * 0.5
-            )
+            radius = np.linalg.norm(coord) + 0.5*self.tm.get_tol(number, number)
             if radius > r_max:
                 r_max = radius
         self.radius = r_max
@@ -338,21 +396,108 @@ class pyxtal_molecule:
     def get_symbols(self):
         self.symbols = [specie.name for specie in self.mol.species]
 
-    def get_tols_matrix(self):
+    def get_tols_matrix(self, mol2=None, tm=None):
         """
-        Returns: a 2D matrix which is used internally for distance checking.
+        Compute the 2D tolerance matrix between the current and other molecules
+
+        Args:
+            mol2: the 2nd pyxtal_molecule object
+            tm: tolerance class
+
+        Returns:
+            a 2D matrix which is used internally for distance checking.
         """
-        numbers = self.mol.atomic_numbers
-        tols = np.zeros((len(numbers), len(numbers)))
-        for i1, number1 in enumerate(numbers):
-            for i2, number2 in enumerate(numbers):
-                tols[i1][i2] = self.tm.get_tol(number1, number2)
+        if tm is None:
+            tm = self.tm
+
+        numbers1 = self.mol.atomic_numbers
+        if mol2 is None:
+            numbers2 = self.mol.atomic_numbers
+        else:
+            numbers2 = mol2.mol.atomic_numbers
+
+        tols = np.zeros((len(numbers1), len(numbers2)))
+        for i1, number1 in enumerate(numbers1):
+            for i2, number2 in enumerate(numbers2):
+                tols[i1, i2] = tm.get_tol(number1, number2)
                 # allow hydrogen bond
                 if [number1, number2] in [[1,7], [1,8], [1,9], [7,1], [8,1], [9,1]]:
-                    tols[i1][i2] *= 0.9
+                    tols[i1, i2] *= 0.9
+
         if len(self.mol)==1:
             tols *= 0.8 # if only one atom, reduce the tolerance
-        self.tols_matrix = tols
+        return tols
+
+    def get_coefs_matrix(self, mol2=None):
+        """
+        Get the Atom-Atom potential parameters
+        E = A*exp(-B*R) - C*R^(-6)
+        according to Gavezotti, Acc. Chem. Res., 27, 1994
+        in Kcal/mol and angstrom
+
+        Args:
+            mol2: the 2nd pyxtal_molecule object
+            tm: tolerance class
+
+        Returns:
+            a 3D matrix for computing the intermolecular energy
+        """
+        numbers1 = self.mol.atomic_numbers
+        if mol2 is None:
+            numbers2 = self.mol.atomic_numbers
+        else:
+            numbers2 = mol2.mol.atomic_numbers
+
+        coefs = np.zeros([len(numbers1), len(numbers2), 3])
+        for i1, n1 in enumerate(numbers1):
+            for i2, n2 in enumerate(numbers2):
+                if [n1, n2] in [[1,1]]:              #H-H
+                    coefs[i1, i2, :] = [5774, 4.01, 26.1]
+                elif [n1, n2] in [[1,6], [6,1]]:     #H-C
+                    coefs[i1, i2, :] = [28870, 4.10, 113.]
+                elif [n1, n2] in [[1,7], [7,1]]:     #H-N
+                    coefs[i1, i2, :] = [54560, 4.52, 120.]
+                elif [n1, n2] in [[1,8], [8,1]]:     #H-O
+                    coefs[i1, i2, :] = [70610, 4.82, 105.]
+                elif [n1, n2] in [[1,16],[16,1]]:    #H-S
+                    coefs[i1, i2, :] = [64190, 4.03, 279.]
+                elif [n1, n2] in [[1,17],[17,1]]:    #H-Cl
+                    coefs[i1, i2, :] = [70020, 4.09, 279.]
+                elif [n1, n2] in [[6,6]]:            #C-C
+                    coefs[i1, i2, :] = [54050, 3.47, 578.]
+                elif [n1, n2] in [[6,7], [7,6]]:     #C-N
+                    coefs[i1, i2, :] = [117470, 3.86, 667.]
+                elif [n1, n2] in [[6,8], [8,6]]:     #C-O
+                    coefs[i1, i2, :] = [93950, 3.74, 641.]
+                elif [n1, n2] in [[6,16], [16,6]]:   #C-S
+                    coefs[i1, i2, :] = [126460, 3.41, 1504.]
+                elif [n1, n2] in [[6,8], [8,6]]:     #C-Cl
+                    coefs[i1, i2, :] = [93370, 3.52, 923.]
+                elif [n1, n2] in [[7,7]]:            #N-N
+                    coefs[i1, i2, :] = [87300, 3.65, 691.]
+                elif [n1, n2] in [[7,8], [8,7]]:     #N-O
+                    coefs[i1, i2, :] = [64190, 3.86, 364.]
+                elif [n1, n2] in [[8,8]]:            #O-O
+                    coefs[i1, i2, :] = [46680, 3.74, 319.]
+                elif [n1, n2] in [[8,16], [16,8]]:   #O-S
+                    coefs[i1, i2, :] = [110160, 3.63, 906.]
+                elif [n1, n2] in [[8,17], [17,8]]:   #O-Cl
+                    coefs[i1, i2, :] = [80855, 3.63, 665.]
+                elif [n1, n2] in [[16, 16]]:         #S-S
+                    coefs[i1, i2, :] = [259960, 3.52, 2571]
+                elif [n1, n2] in [[16, 17], [17, 16]]: #S-Cl
+                    coefs[i1, i2, :] = [1800000, 3.52, 2000]   #made
+                elif [n1, n2] in [[17, 17]]:          #Cl-Cl
+                    coefs[i1, i2, :] = [140050, 3.52, 1385]
+                #todo add H-bonds
+                #HB-O(amides):   3607810, 7.78, 238
+                #HB-O(acids):    6313670, 8.75, 205
+                #HB-O(alcohols): 4509750, 7.78, 298
+                #HB-N(-N-H-N):   7215600, 7.78, 476
+                #HB-N(-NH2-N):   1803920, 7.37, 165
+                else:
+                    return None
+        return coefs
 
     def show(self):
         """
@@ -397,7 +542,7 @@ class pyxtal_molecule:
         from rdkit.Chem import rdMolTransforms as rdmt
 
         if smile not in ["Cl-", "F-", "Br-", "I-", "Li+", "Na+"]:
-            torsionlist = find_id_from_smile(smile)
+            torsionlist = find_rotor_from_smile(smile)
             mol = Chem.MolFromSmiles(smile)
             mol = Chem.AddHs(mol)
             symbols = []
@@ -408,8 +553,11 @@ class pyxtal_molecule:
                 cid = 0
             else:
                 ps = AllChem.ETKDGv3()
-                ps.randomSeed=self.seed
+                ps.randomSeed = self.seed
                 AllChem.EmbedMultipleConfs(mol, 1, ps)
+                if mol.GetNumConformers() == 0:
+                    AllChem.EmbedMultipleConfs(mol, 3, ps)
+
             res = AllChem.MMFFOptimizeMoleculeConfs(mol)
             engs = [c[1] for c in res]
             cid = engs.index(min(engs))
@@ -420,10 +568,10 @@ class pyxtal_molecule:
             if fix or torsions is not None or len(torsionlist)==0:
                 conf = ref_conf
             else:
-                AllChem.EmbedMultipleConfs(mol, 
-                                           numConfs=max([1, 4*len(torsionlist)]), 
-                                           maxAttempts=200, 
-                                           useRandomCoords=True, 
+                AllChem.EmbedMultipleConfs(mol,
+                                           numConfs=max([1, 4*len(torsionlist)]),
+                                           maxAttempts=200,
+                                           useRandomCoords=True,
                                            pruneRmsThresh=0.5)
                 N_confs = mol.GetNumConformers()
                 conf_id = choice(range(N_confs))
@@ -450,7 +598,7 @@ class pyxtal_molecule:
         slightly perturb the torsion
         """
         angs = self.get_torsion_angles(xyz, self.torsionlist)
-        angs *= (1+0.1*np.random.uniform(-1., 1., len(angs))) 
+        angs *= (1+0.1*np.random.uniform(-1., 1., len(angs)))
         xyz = self.set_torsion_angles(conf, angs, torsionlist=self.torsionlist)
         xyz -= self.get_center(xyz)
         return xyz
@@ -463,15 +611,15 @@ class pyxtal_molecule:
         from rdkit.Chem import rdMolTransforms as rdmt
 
         # Rotation
-        if len(self.smile) > 1: 
+        if len(self.smile) > 1:
             trans = rdmt.ComputeCanonicalTransform(conf)
-            if abs(abs(np.linalg.det(trans))-1.0)>1e-1: 
+            if abs(abs(np.linalg.det(trans))-1.0)>1e-1:
                 print("Bug in trans", np.linalg.det(trans))
                 import sys; sys.exit()
-            elif np.linalg.det(trans[:3,:3]) < 0: 
+            elif np.linalg.det(trans[:3,:3]) < 0:
                 trans[:3,:3] *= -1
 
-            # add reflection if needed    
+            # add reflection if needed
             if reflect: trans[:3,:3] *= -1
             rdmt.TransformConformer(conf, trans)
 
@@ -507,7 +655,7 @@ class pyxtal_molecule:
                     # from rdkit
                     from rdkit.Geometry import Point3D
                     from rdkit.Chem import rdMolTransforms as rdmt
-                    
+
                     conf = self.rdkit_mol().GetConformer(0)
                     for i in range(len(xyz)):
                         x, y, z = xyz[i]
@@ -516,11 +664,11 @@ class pyxtal_molecule:
 
                     return np.array([pt.x, pt.y, pt.z])
 
-    def get_principle_axes(self, xyz):
+    def get_principle_axes(self, xyz, rdmt=True):
         """
         get the principle axis for a rotated xyz
         """
-        if self.smile is None or len(self.smile)==1: 
+        if self.smile is None or len(self.smile)==1 or not rdmt:
             Inertia = get_inertia_tensor(xyz)
             _, matrix = np.linalg.eigh(Inertia)
             return matrix
@@ -535,13 +683,14 @@ class pyxtal_molecule:
 
             return rdmt.ComputePrincipalAxesAndMoments(conf1)[0]
 
-    def get_torsion_angles(self, xyz, torsionlist=None):
+    def get_torsion_angles(self, xyz=None, torsionlist=None):
         """
         get the torsion angles
         """
         from rdkit.Geometry import Point3D
         from rdkit.Chem import rdMolTransforms as rdmt
 
+        if xyz is None: xyz = self.mol.cart_coords
         if torsionlist is None: torsionlist=self.torsionlist
 
         angs = []
@@ -555,7 +704,7 @@ class pyxtal_molecule:
                 (i, j, k, l) = torsion
                 angs.append(rdmt.GetDihedralDeg(conf, i, j, k, l))
         return angs
-    
+
     def set_torsion_angles(self, conf, angles, reflect=False, torsionlist=None):
         """
         reset the torsion angles and update molecular xyz
@@ -566,16 +715,83 @@ class pyxtal_molecule:
         for id, torsion in enumerate(torsionlist):
             (i, j, k, l) = torsion
             rdmt.SetDihedralDeg(conf, i, j, k, l, angles[id])
-        
+
         xyz = self.align(conf, reflect)
         return xyz
 
-    def get_rmsd(self, xyz):
+    def relax(self, xyz, align=False):
+        """
+        Relax the input xyz coordinates with rdit MMFF
+
+        Args:
+            xyz: 3D coordinates
+            align: whether or not align the xyz
+
+        Returns:
+            xyz: new xyz
+            eng: energy value
+        """
+
+        from rdkit.Chem import AllChem
+        from rdkit.Geometry import Point3D
+
+        mol = self.rdkit_mol(1)
+        conf0 = mol.GetConformer(0)
+        # reset the xyz
+        for i in range(len(self.mol)):
+            x,y,z = xyz[i]
+            conf0.SetAtomPosition(i,Point3D(x,y,z))
+        res = AllChem.MMFFOptimizeMoleculeConfs(mol)
+        if align:
+            xyz = self.align(conf0)
+        else:
+            xyz = mol.GetConformer(0).GetPositions()
+        return xyz, res[0][1]
+
+
+    def get_rmsd2(self, xyz0, xyz1):
         """
         Compute the rmsd with another 3D xyz coordinates
 
         Args:
             xyz: 3D coordinates
+
+        Returns:
+            rmsd:
+            transition matrix:
+            match:
+        """
+
+        from rdkit import Chem
+        from rdkit.Geometry import Point3D
+        from rdkit.Chem import rdMolAlign, RemoveHs
+
+        mol = self.rdkit_mol(3)
+        conf0 = mol.GetConformer(0)
+        conf1 = mol.GetConformer(1) 
+        for i in range(len(self.mol)):
+            x0,y0,z0 = xyz0[i]
+            x1,y1,z1 = xyz1[i]
+            conf0.SetAtomPosition(i,Point3D(x0,y0,z0))
+            conf1.SetAtomPosition(i,Point3D(x1,y1,z1))
+
+        mol = RemoveHs(mol)
+        rmsd, trans = rdMolAlign.GetAlignmentTransform(mol, mol, 1, 0)
+        
+        return rmsd, trans
+
+
+    def get_rmsd(self, xyz, debug=False):
+        """
+        Compute the rmsd with another 3D xyz coordinates
+
+        Args:
+            xyz: 3D coordinates
+
+        Returns:
+            rmsd:
+            transition matrix:
+            match:
         """
 
         from rdkit import Chem
@@ -590,7 +806,7 @@ class pyxtal_molecule:
         angs = self.get_torsion_angles(xyz)
         xyz0 = self.set_torsion_angles(conf0, angs) #conf0 with aligned
         xyz1 = self.set_torsion_angles(conf0, angs, True) #conf0 with aligned+reflect
-
+        #print('xyz0', xyz0)
         # reset the xyz
         for i in range(len(self.mol)):
             x0,y0,z0 = xyz0[i]
@@ -603,14 +819,19 @@ class pyxtal_molecule:
         mol = RemoveHs(mol)
         rmsd1, trans1 = rdMolAlign.GetAlignmentTransform(mol, mol, 2, 0)
         rmsd2, trans2 = rdMolAlign.GetAlignmentTransform(mol, mol, 2, 1)
-        #rdmolfiles.MolToXYZFile(mol, '1.xyz', 0)
-        #rdmolfiles.MolToXYZFile(mol, '2.xyz', 1)
-        #rdmolfiles.MolToXYZFile(mol, '3.xyz', 2)
-       
+        
+        if debug:
+            from rdkit.Chem import rdmolfiles
+            rdmolfiles.MolToXYZFile(mol, '1.xyz', 0)
+            rdmolfiles.MolToXYZFile(mol, '2.xyz', 1)
+            rdmolfiles.MolToXYZFile(mol, '3.xyz', 2)
+            print(rmsd1, rmsd2)
         if rmsd1 <= rmsd2:
-            return rmsd1, trans1, True
+            #return rmsd1, trans1, True
+            return rmsd1, trans1, False
         else:
-            return rmsd2, trans2, False
+            #return rmsd2, trans2, False
+            return rmsd2, trans2, True
 
     def get_orientation(self, xyz, rtol=0.15):
         """
@@ -619,13 +840,13 @@ class pyxtal_molecule:
         Args:
             xyz: molecular xyz
         """
-        
+
         xyz -= self.get_center(xyz)
 
         if len(self.smile) > 1: # not in ["O", "o"]:
             rmsd, trans, reflect = self.get_rmsd(xyz)
             tol = rtol*len(xyz)
-            
+
             if rmsd < tol:
                 trans = trans[:3,:3].T
                 r = Rotation.from_matrix(trans)
@@ -634,16 +855,16 @@ class pyxtal_molecule:
                 msg = "Problem in conformer\n"
                 msg += "{:5.2f} {:5.2f}\n".format(rmsd1, rmsd2)
                 if len(self.torsionlist) > 0:
-                    msg += str(self.get_torsion_angles(xyz)) + '\n'  
-                    msg += str(self.get_torsion_angles(xyz0)) + '\n'  
-                    msg += str(self.get_torsion_angles(xyz1)) + '\n'  
+                    msg += str(self.get_torsion_angles(xyz)) + '\n'
+                    msg += str(self.get_torsion_angles(xyz0)) + '\n'
+                    msg += str(self.get_torsion_angles(xyz1)) + '\n'
                 raise ConformerError(msg)
         else:
             # the orientation of CH4, NH3, H2O
             ref = np.array([[-0.00111384,  0.36313718,  0.        ],
-                            [-0.82498189, -0.18196256,  0.        ], 
+                            [-0.82498189, -0.18196256,  0.        ],
                             [ 0.82609573, -0.18117463,  0.        ]])
-            Inertia = get_inertia_tensor(xyz) 
+            Inertia = get_inertia_tensor(xyz)
             _, matrix = np.linalg.eigh(Inertia)
 
             ref0 = np.dot(xyz, matrix)
@@ -695,22 +916,33 @@ class pyxtal_molecule:
         xyz *= -1
         self.reset_positions(xyz)
 
-    def get_symmetry(self):
+    def get_symmetry(self, xyz=None, symmetrize=False, rtol=0.30):
         """
         Set the molecule's point symmetry.
             - pga: pymatgen.symmetry.analyzer.PointGroupAnalyzer object
             - pg: pyxtal.symmetry.Group object
             - symops: a list of SymmOp objects
-        """
 
-        mol = deepcopy(self.mol)
+        Args:
+            symmetrize: boolean, whether or not symmetrize the coordinates
+        """
+        if xyz is None:
+            mol = deepcopy(self.mol)
+        else:
+            mol = Molecule(self.symbols, xyz)
+
         if self.smile is not None:
             mol.remove_species('H')
             mol._spin_multiplicity = None #don't check spin
-        pga = PointGroupAnalyzer(mol)#0.3)
+
+        if symmetrize:
+            pga = PointGroupAnalyzer(mol, rtol, eigen_tolerance=1e-3)
+            mol = pga.symmetrize_molecule()["sym_mol"]
+        pga = PointGroupAnalyzer(mol, rtol, eigen_tolerance=1e-3)
         self.mol_no_h = mol
 
-        # For single atoms, we cannot represent the point group using a list of operations
+        #print(mol.to(fmt='xyz'), pga.sch_symbol)
+        # For single atoms, no point group using a list of operations
         if len(mol) == 1:
             symm_m = []
             symbol = 'C1'
@@ -718,7 +950,7 @@ class pyxtal_molecule:
             symbol = pga.sch_symbol
             pg = pga.get_pointgroup()
             symm_m = [op for op in pg]
-    
+
             if "*" in symbol: # linear molecules
                 symbol = symbol.replace('*','6')
                 # Add 12-fold  and reflections in place of ininitesimal rotation
@@ -744,21 +976,22 @@ class pyxtal_molecule:
                             symm_m.append(SymmOp.from_xyz_string("-x,y,z"))
                             symm_m.append(SymmOp.from_xyz_string("x,-y,z"))
                             #r = SymmOp.from_xyz_string("x,-y,-z")
-                        # Generate a full list of SymmOps for the molecule's pointgroup
+                        # Generate a full list of SymmOps for the pointgroup
                         symm_m = generate_full_symmops(symm_m, 1e-3)
                         break
         self.symops = symm_m
         self.pga = pga
+        if symbol == 'S2': symbol = 'Ci'
         self.pg = Group(symbol, dim=0)
 
     def get_orientations_in_wps(self, wps=None, rtol=1e-2):
         """
         Compute the valid orientations from a given Wyckoff site symmetry.
-    
+
         Args:
             wp: a pyxtal.symmetry.Wyckoff_position object
         Returns:
-            a list of operations.Orientation objects 
+            a list of operations.Orientation objects
         """
         if wps is None:
             return None, True
@@ -775,11 +1008,11 @@ class pyxtal_molecule:
     def get_orientations_in_wp(self, wp, rtol=1e-2):
         """
         Compute the valid orientations from a given Wyckoff site symmetry.
-    
+
         Args:
             wp: a pyxtal.symmetry.Wyckoff_position object
         Returns:
-            a list of pyxtal.molecule.Orientation objects 
+            a list of pyxtal.molecule.Orientation objects
         """
         # For single atoms, there are no constraints
         if len(self.mol) == 1 or wp.index == 0:
@@ -788,28 +1021,30 @@ class pyxtal_molecule:
         elif wp.index > 1 and self.pga.sch_symbol == 'C1':
             return []
 
-        symm_w = wp.symmetry_m[0]
+        symm_w = wp.get_site_symm_wo_translation() #symmetry without translation
         # molecule has fewer symops
         if len(self.pg[0]) < len(symm_w):
             return []
-    
-        symm_m = self.symops 
+
+        symm_m = self.symops
         wyckoffs = wp.ops
         opa_m = []
         for op_m in symm_m:
             opa = OperationAnalyzer(op_m)
             opa_m.append(opa)
-        
+
         # Store OperationAnalyzer objects for each Wyckoff symmetry SymmOp
         opa_w = []
         for op_w in symm_w:
             opa_w.append(OperationAnalyzer(op_w))
-    
-        # Check for constraints from the Wyckoff symmetry
-        # If we find ANY two constraints (SymmOps with unique axes), the molecule's
-        # point group MUST contain SymmOps which can be aligned to these particular
-        # constraints. However, there may be multiple compatible orientations of the
-        # molecule consistent with these constraints
+
+        """
+        Check for constraints from the Wyckoff symmetry. If we find ANY two
+        constraints (SymmOps with unique axes), the molecule's point group MUST
+        contain SymmOps which can be aligned to these particular constraints.
+        However, there may be multiple compatible orientations of the molecule
+        consistent with these constraints
+        """
         constraint1 = None
         constraint2 = None
         for i, op_w in enumerate(symm_w):
@@ -835,9 +1070,11 @@ class pyxtal_molecule:
                     constraints_m.append([opa1, []])
                     # Generate 2nd constraint in opposite direction
                     extra = deepcopy(opa1)
-                    extra.axis = [opa1.axis[0] * -1, opa1.axis[1] * -1, opa1.axis[2] * -1]
+                    extra.axis = [opa1.axis[0] * -1,
+                                  opa1.axis[1] * -1,
+                                  opa1.axis[2] * -1]
                     constraints_m.append([extra, []])
-    
+
         # Remove redundancy for the first constraints
         list_i = list(range(len(constraints_m)))
         list_j = list(range(len(constraints_m)))
@@ -871,7 +1108,7 @@ class pyxtal_molecule:
         constraints_m = []
         for i in list_i:
             constraints_m.append(c_m[i])
-    
+
         # Generate 2nd consistent molecular constraints
         valid = list(range(len(constraints_m)))
         if constraint2 is not None:
@@ -898,7 +1135,7 @@ class pyxtal_molecule:
         constraints_m = []
         for i in valid:
             constraints_m.append(copy[i])
-    
+
         # Generate orientations consistent with the possible constraints
         orientations = []
         # Loop over molecular constraint sets
@@ -954,8 +1191,8 @@ class pyxtal_molecule:
         orientations_new = []
         for i in list_i:
             orientations_new.append(orientations[i])
-    
-        #Check each of the found orientations for consistency with the Wyckoff pos.
+
+        #Check each of the found orientations for consistency with Wyckoff site.
         #If consistent, put into an array of valid orientations
         #print("======", orientations_new)
         allowed = []
@@ -968,9 +1205,17 @@ class pyxtal_molecule:
                 allowed.append(o)
         return allowed
 
+    def get_energy(self, xyz1, xyz2):
+        """
+        Get packing energy between two neighboring molecules
+        """
+        dists = cdist(xyz1-xyz2)
+
+
+
 class Box:
     """
-    Class for storing the binding box for a molecule. 
+    Class for storing the binding box for a molecule.
 
     Args:
         dims: [length, width, height]
@@ -1019,7 +1264,7 @@ class Orientation:
 
     def __init__(self, matrix=None, degrees=2, axis=None):
         self.matrix = np.array(matrix)
-        self.degrees = degrees  # The number of degrees of freedom.
+        self.degrees = degrees
         if degrees == 1:
             if axis is None:
                 raise ValueError("axis is required for orientation")
@@ -1027,7 +1272,7 @@ class Orientation:
                 axis /= np.linalg.norm(axis)
         self.axis = axis
 
-        self.r = Rotation.from_matrix(self.matrix)  # scipy transform.Rotation class
+        self.r = Rotation.from_matrix(self.matrix)
         self.angle = None
 
     def __str__(self):
@@ -1068,8 +1313,8 @@ class Orientation:
 
     def change_orientation(self, angle="random", flip=False):
         """
-        Allows for specification of an angle (possibly random) to
-        rotate about the constraint axis.
+        Allows for specification of an angle (possibly random) to rotate about
+        the constraint axis.
 
         Args:
             angle: an angle to rotate about the constraint axis.
@@ -1160,9 +1405,9 @@ class Orientation:
 
     def get_op(self): #, angle=None):
         """
-        Generate a SymmOp object consistent with the orientation's
-        constraints. Allows for specification of an angle (possibly random) to
-        rotate about the constraint axis.
+        Generate a SymmOp object consistent with the orientation's constraints.
+        Allows for specification of an angle (possibly random) to rotate about
+        the constraint axis.
 
         Args:
             angle: an angle to rotate about the constraint axis. If "random",
@@ -1198,8 +1443,7 @@ class Orientation:
 
 def get_inertia_tensor(coords, weights=None):
     """
-    Calculate the symmetric inertia tensor for a Molecule
-    the principal axes of symmetry.
+    Calculate the symmetric inertia tensor for a molecule.
 
     Args:
         coords: [N, 3] array of coordinates
@@ -1222,8 +1466,7 @@ def get_inertia_tensor(coords, weights=None):
 
 def reoriented_molecule(mol): #, nested=False):
     """
-    Reorient a molecule so that its principal axes are aligned with the
-    identity matrix.
+    Allign a molecule so that its principal axes is the identity matrix.
 
     Args:
         mol: a Molecule object
@@ -1238,7 +1481,7 @@ def reoriented_molecule(mol): #, nested=False):
     A = get_inertia_tensor(coords)
     # Store the eigenvectors of the inertia tensor
     P = np.linalg.eigh(A)[1]
-    if np.linalg.det(P) < 0: P[0] *= -1
+    if np.linalg.det(P) < 0: P[:,0] *= -1
     coords = np.dot(coords, P)
     return Molecule(numbers, coords), P
 
@@ -1254,9 +1497,8 @@ def is_compatible_symmetry(mol, wp):
     # For single atoms, there are no constraints
     if len(mol) == 1 or wp.index == 0:
         return True
-
     pga = PointGroupAnalyzer(mol)
-    for op in wp.symmetry_m[0]:
+    for op in wp.get_site_symm_wo_translation(): #symmetry without translation
         #print("XXXX", pga.is_valid_op(op), op.as_xyz_string())
         if not pga.is_valid_op(op):
             return False
