@@ -3,7 +3,7 @@ import numpy as np
 from pyxtal.symmetry import Group
 from pyxtal.lattice import Lattice
 from pyxtal.wyckoff_site import mol_site
-from pyxtal.molecule import find_id_from_smile
+from pyxtal.molecule import find_rotor_from_smile
  
 class representation():
     """
@@ -30,14 +30,18 @@ class representation():
         return self.to_string()
 
     @classmethod
-    def from_pyxtal(cls, struc):
+    def from_pyxtal(cls, struc, standard=False):
         """
         Initialize 1D rep. from the pyxtal object
 
         Args:
             struc: pyxtal object
         """
-        symmetry = [struc.group.number, struc.diag]
+        if standard and not struc.standard_setting:
+            #struc.optimize_lattice(standard=True)
+            pmg = struc.to_pymatgen()
+            struc.from_seed(pmg, molecules=struc.molecules, standard=True)
+        symmetry = [struc.mol_sites[0].wp.hall_number]
         lat = struc.lattice.encode()
         vector = [symmetry + lat]
         smiles = []
@@ -61,21 +65,21 @@ class representation():
             composition = [1] * len(smiles)
 
         inputs = [float(tmp) for tmp in inputs.split()]
-        g, diag = int(inputs[0]), int(inputs[1])
-        if g <= 2:
+        hn = int(inputs[0])
+        if hn <= 2:
             n_cell = 8
-        elif g <= 15:
+        elif hn <= 107:
             n_cell = 6
-        elif g <= 74:
+        elif hn <= 348:
             n_cell = 5
-        elif g <= 194:
+        elif hn <= 488:
             n_cell = 4
         else:
             n_cell = 3 #cubic
-        cell = [g, diag] + inputs[2:n_cell]
+        cell = [hn] + inputs[1:n_cell-1]
         
         x = [cell]
-        n_site = int(inputs[n_cell])
+        n_site = int(inputs[n_cell-1])
         if n_site != sum(composition):
             msg = "Composition is inconsistent: {:d}/{:d}\n".format(sum(composition), n_site)
             msg += str(inputs)
@@ -89,12 +93,17 @@ class representation():
                 if smile in ["Cl-"]:
                     n_mol = 4
                 else:
-                    n_torsion = len(find_id_from_smile(smile))
+                    n_torsion = len(find_rotor_from_smile(smile))
                     n_mol = 7 + n_torsion
-                inputs[n_cell+n_mol-1] = int(inputs[n_cell+n_mol-1])
-                x.append(inputs[n_cell:n_cell+n_mol])
+                #inversion
+                inputs[n_cell+n_mol-2] = int(inputs[n_cell+n_mol-2])
+                x.append(inputs[n_cell-1:n_cell+n_mol-1])
                 n_cell += n_mol
         return cls(x, smiles)
+
+    def to_standard_setting(self):
+        xtal = self.to_pyxtal()
+        self.x = representation.from_pyxtal(xtal, standard=True).x
  
     def to_pyxtal(self, smiles=None, composition=None):
         """
@@ -120,22 +129,22 @@ class representation():
         # symmetry
         v = self.x[0]
         struc = pyxtal(molecular=True)
-        struc.group, number, struc.diag = Group(v[0]), v[0], v[1]
+        struc.group, number = Group(v[0], use_hall=True), v[0]
     
         # lattice
         ltype = struc.group.lattice_type
         if ltype == 'triclinic':
-            a, b, c, alpha, beta, gamma = v[2], v[3], v[4], v[5], v[6], v[7]
+            a, b, c, alpha, beta, gamma = v[1], v[2], v[3], v[4], v[5], v[6]
         elif ltype == 'monoclinic':
-            a, b, c, alpha, beta, gamma = v[2], v[3], v[4], 90, v[5], 90
+            a, b, c, alpha, beta, gamma = v[1], v[2], v[3], 90, v[4], 90
         elif ltype == 'orthorhombic':
-            a, b, c, alpha, beta, gamma = v[2], v[3], v[4], 90, 90, 90
+            a, b, c, alpha, beta, gamma = v[1], v[2], v[3], 90, 90, 90
         elif ltype == 'tetragonal':
-            a, b, c, alpha, beta, gamma = v[2], v[2], v[3], 90, 90, 90
+            a, b, c, alpha, beta, gamma = v[1], v[1], v[2], 90, 90, 90
         elif ltype == 'hexagonal':
-            a, b, c, alpha, beta, gamma = v[2], v[2], v[3], 90, 90, 120
+            a, b, c, alpha, beta, gamma = v[1], v[1], v[2], 90, 90, 120
         else:
-            a, b, c, alpha, beta, gamma = v[2], v[2], v[2], 90, 90, 90
+            a, b, c, alpha, beta, gamma = v[1], v[1], v[1], 90, 90, 90
         try:
             struc.lattice = Lattice.from_para(a, b, c, alpha, beta, gamma, ltype=ltype)
         except:
@@ -155,10 +164,11 @@ class representation():
                 v = self.x[count]
                 dicts = {}
                 dicts['smile'] = smile
+                dicts['type'] = i
                 dicts['dim'] = 3
                 dicts['PBC'] = [1, 1, 1]
-                dicts['number'] = number
-                dicts['diag'] = struc.diag
+                #dicts['number'] = number
+                dicts['hn'] = struc.group.hall_number
                 dicts['index'] = 0
                 dicts['lattice'] = struc.lattice.matrix
                 dicts['lattice_type'] = ltype
@@ -168,6 +178,7 @@ class representation():
                     dicts['rotor'] = v[6:-1]
                     dicts['reflect'] = int(v[-1])
                 site = mol_site.from_1D_dicts(dicts)
+                site.type = i
                 struc.mol_sites.append(site)
                 struc.numMols[i] += site.wp.multiplicity
                 #move to next rep
@@ -177,6 +188,7 @@ class representation():
         struc._get_formula()
         struc.source = '1D rep.'
         struc.valid = True
+        struc.standard_setting = site.wp.is_standard_setting()
 
         return struc
     
@@ -190,17 +202,17 @@ class representation():
             tag: string
         """
         x = self.x
-        strs = "{:3d} {:d} ".format(int(x[0][0]), int(x[0][1]))
+        strs = "{:3d} ".format(int(x[0][0]))
 
         # data for cell
-        if x[0][0] <= 74:
-            num = 5
-        elif x[0][0] <=194:
+        if x[0][0] <= 348:
             num = 4
-        else:
+        elif x[0][0] <= 488:
             num = 3
+        else: #cubic
+            num = 2
 
-        for c in x[0][2:num]:
+        for c in x[0][1:num]:
             strs += "{:5.2f} ".format(c)
         for c in x[0][num:]:
             strs += "{:5.1f} ".format(c)
@@ -218,20 +230,77 @@ class representation():
             strs += "{:5.2f}".format(time)
 
         if eng is not None:
-            strs += "{:10.2f}".format(eng)
+            strs += "{:11.3f}".format(eng)
     
         if tag is not None:
             strs += " {:s}".format(tag)
     
         return strs
 
-
+    def same_smiles(self, smiles):
+        if len(self.smiles) == smiles:
+            for s1, s2 in zip(self.smiles, smiles):
+                if s2 != s2:
+                    return False
+            return True
+        else:
+            return False
+            
+    def get_dist(self, rep):
+        """
+        get distance with the other rep1
+        Now only supports Z'=1
+        """
+        from pyxtal.symmetry import Wyckoff_position as WP
+        if self.same_smiles(rep.smiles):
+            msg = 'different smiles'
+            print(msg)
+            return None
+        elif len(self.x) != len(rep.x):
+            msg = 'different number of sites'
+            print(msg)
+            return None
+        elif self.x[0][0] != rep.x[0][0]:
+            msg = 'different space group numbers'
+            print(msg)
+            return None
+        else:
+            diffs = []
+            wp = WP.from_group_and_index(self.x[0][0], 0, use_hall=True)
+            for i in range(len(self.x)):
+                diff = np.zeros(len(self.x[i]))
+                tmp1 = np.array(self.x[i])
+                tmp2 = np.array(rep.x[i])
+                # cell difference
+                if i == 0:
+                    diff_cell = tmp2 - tmp1
+                    diffs.extend(diff_cell)
+                # site difference
+                else:
+                    # symmmetry variation
+                    xyzs = wp.apply_ops(tmp2[:3])
+                    diff_xyzs = xyzs - tmp1[:3]
+                    diff_xyzs -= np.round(diff_xyzs)
+                    id = np.argmin(np.linalg.norm(diff_xyzs, axis=1))
+                    diff_xyz = diff_xyzs[id]
+                    diff_ori = tmp2[3:6] - tmp1[3:6]
+                    diff_ori /= [360.0, 180.0, 360.0]
+                    diff_ori -= np.round(diff_ori)
+                    diff_ori *= [360.0, 180.0, 360.0]
+                    diff_tor = tmp2[6:] - tmp1[6:]
+                    diff_tor /= 360.0
+                    diff_tor -= np.round(diff_tor)
+                    diff_tor *= 360.0
+                    diffs.extend(diff_xyz)
+                    diffs.extend(diff_ori)
+                    diffs.extend(diff_tor)
+            return np.array(diffs)
 
 if __name__ == "__main__":
 
     #aspirin
     smiles = ['CC(=O)OC1=CC=CC=C1C(=O)O']
-    x = [[14,False,11.43,6.49,11.19,83.31],[0.77,0.57,0.53,48.55,24.31,145.94,-77.85,-4.40,170.86,False]]
+    x = [[81,11.43,6.49,11.19,83.31],[0.77,0.57,0.53,48.55,24.31,145.94,-77.85,-4.40,170.86,False]]
     #rep0 = representation(x, smiles)
     #print(rep0.to_string())
     rep1 = representation(x, smiles)
@@ -240,6 +309,38 @@ if __name__ == "__main__":
     rep2 = representation.from_pyxtal(xtal)
     print(rep2.to_pyxtal())
     print(rep2.to_string())
-    string = "14 0 11.43  6.49 11.19 83.31 1  0.77  0.57  0.53 48.55 24.31 145.9 -77.85 -4.40 170.9 0"
+    string = "82 11.43  6.49 11.19 83.31 1  0.77  0.57  0.53 48.55 24.31 145.9 -77.85 -4.40 170.9 0"
     rep3 = representation.from_string(string, smiles)
+    print(rep3.to_string())
     print(rep3.to_pyxtal())
+    rep3.to_standard_setting()
+    print(rep3.to_pyxtal())
+    print(rep3.to_string())
+
+    string1 = "81 14.08  6.36 25.31  83.9 1 0.83 0.40 0.63  136.6  -21.6 -151.1 -101.1 -131.2  154.7 -176.4 -147.8  178.2 -179.1  -53.3 0"
+    string2 = "81 14.08  6.36 25.31  83.9 1 0.03 0.84 0.89  149.1   -8.0  -37.8  -39.9 -104.2  176.2 -179.6  137.8 -178.5 -173.3 -103.6 0"
+    smiles = ['CC1=CC=C(C=C1)S(=O)(=O)C2=C(N=C(S2)C3=CC=C(C=C3)NC(=O)OCC4=CC=CC=C4)C']
+    rep4 = representation.from_string(string1, smiles)
+    rep5 = representation.from_string(string2, smiles)
+    print(string1)
+    print(string2)
+    print(rep4.get_dist(rep5))
+
+    #strings = [
+    #"83 14.08  6.36 25.31  83.9 1 0.72 0.40 0.27  131.6  -17.0 -120.0  -83.8 -134.1 -174.5 -175.7 -168.8  173.9  178.0 -157.4 0",
+    #"81 14.08  6.36 25.31  83.9 1 0.59 0.81 0.39 -117.8  -50.1  -95.3  -25.8  -80.6  164.7  155.9 -124.9 -159.2  178.6 -154.7 0",
+    #"81 14.08  6.36 25.31  83.9 1 0.75 0.09 0.01  133.8  -19.5  -55.1  -86.7  -91.7 -175.0 -170.4 -176.8  173.3 -164.8  -58.4 0",
+    #"81 14.08  6.36 25.31  83.9 1 0.72 0.44 0.01  135.2   27.5   97.2 -101.1 -105.1  -29.7 -169.7  -50.1  172.2 -173.1  131.6 0",
+    #"82 14.00  6.34 25.26  83.6 1 0.21 0.08 0.54  146.0  -12.0   50.2  108.0  112.3 -166.3 -158.7  -35.5  172.3 -168.7  133.0 0",
+    #"81 14.08  6.36 25.31  83.9 1 0.05 0.30 0.89  -68.2   41.2  148.8  -66.9  -85.0 -167.4  172.3 -166.2 -178.3  166.4  -45.9 0",
+    #]
+
+    #import pymatgen.analysis.structure_matcher as sm
+    #matcher = sm.StructureMatcher(ltol=0.3, stol=0.3, angle_tol=10)
+    #for i, string in enumerate(strings):
+    #    print(str(i) + '  ' +string)
+    #    rep4 = representation.from_string(string, smiles)
+    #    pmg1 = rep4.to_pyxtal().to_pymatgen(); pmg1.remove_species('H')
+    #    rep4.to_standard_setting()
+    #    pmg2 = rep4.to_pyxtal().to_pymatgen(); pmg2.remove_species('H')
+    #    print(i, rep4.to_string(), matcher.fit(pmg1, pmg2))
